import sys
import pandas as pd
import numpy as np

class read_pair():
    def __init__(self):
        self.barcode = {1:None,2:None}
        self.site = {1:None,2:None}
        self.seq = {1:None,2:None}
        self.mice = None
        self.type = "N/A"

    def switch_values(self):
        for k in [self.barcode, self.site, self.seq]:
            t = k[1]
            k[1] = k[2]
            k[2] = t

    def proccess(self, mice):
        self.proccessed = True
        self.type = "valid"
        s = sum([i is None for i in self.barcode.values()])
        if s==2:
            self.type = "Not_found"
            return
        elif s==1:
            self.type = "Single_sided"
            return
        elif s != 0:
            raise

        if self.site[1][-1] == "F" and self.site[2][-1] == "R":
            pass
        elif self.site[1][-1] == "R" and self.site[2][-1] == "F":
            self.switch_values()
        elif self.site[1][-1] == self.site[2][-1]:
            if self.site[1][:-1] != self.site[2][:-1]:
                self.type = "inter-site-FF/RR"
            else:
                self.type = "FF/RR"
            return
        else:
            print (self.site)
            raise

        self.mice = mice.loc[self.barcode[1],self.barcode[2]]

        if self.site[1][:-1] != self.site[2][:-1]:
            self.type = "inter-site-FR"
            return

    def __str__(self):
        assert self.proccessed
        return str(self.mice)+ "\t" + self.site[1][:-1] + "\t" + self.seq[1]+"\t"+self.seq[2]

f1 = sys.argv[1]
f2 = sys.argv[2]

primers = pd.read_csv("adapters.tsv",sep="\t", index_col=0, header=None, names=["name","seq","st","end"])
mice = pd.read_csv("mice.tsv",sep="\t", index_col=0, header=0)
barcodes = mice.index.values

assert len(np.unique([len(b) for b in barcodes])) == 1

barcode_len = len(barcodes[0])
primer_len = 10
readsParser = {}

for b in barcodes:
    for primer_name in primers.index.values:
        expected_str = "AC"+b+primers.loc[primer_name].seq[:primer_len]
        assert not expected_str in readsParser.keys()
        readsParser[expected_str] = (b,primer_name)

# read and process data
total = 0
types = {"valid":0, "Not_found":0, "Single_sided":0,
         "inter-site-FF/RR":0, "inter-site-FR":0, "FF/RR":0}
types_by_mice = {}

out = open("processed.txt","w")
with open(f1) as f1:
    with open(f2) as f2:
        for line in f1: # readname
            f2.readline() # readname
            seq1 = f1.readline().strip()
            seq2 = f2.readline().strip()
            pair = read_pair()
            for id,seq in zip([1,2],[seq1,seq2]):
                try:
                    barcode, primer_name = readsParser[seq[:primer_len+barcode_len+2]]
                    pair.barcode[id] = barcode
                    pair.site[id] = primer_name
                    pair.seq[id] = seq
                except KeyError:
                    pass

            pair.proccess(mice=mice)
            if pair.type == "valid":
                out.write(str(pair) + "\n")
            mice_number = -1
            if pair.mice is not None:
                mice_number = pair.mice
            if not mice_number in types_by_mice.keys():
                types_by_mice[mice_number] = dict((k, v) for k, v in types.items())
            types_by_mice[mice_number][pair.type] += 1

            total += 1
            f1.readline() # +
            f2.readline() # +
            f1.readline() # qual
            f2.readline() # qual
            if total % 1000000 == 0 and total >= 1000000:
                print("Processed ", total)

types_by_mice = pd.DataFrame.from_dict(types_by_mice)
types_by_mice = types_by_mice.reindex(sorted(types_by_mice.columns), axis=1)
types_by_mice.to_csv("stats.txt",sep="\t",header=True,index=True)
out.close()
print("Done!")